/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <stdarg.h>
#include <unistd.h>
#include <malloc.h>
#include <gpib/ib.h>
#define HW_GPIB
#include "camera.h"
#include "my_stdio.h"


static int my_ibwrite_generic(
	int (*write_func)(int ud, const void *data, long num_bytes),
	int dev, const char *fmt, ...)
{
	va_list ap;
	int l = -1;
	int n, size = 80;
	char *buff = NULL;

	if ((buff = malloc (size)) == NULL) return l;
	while (1) {
		/* Try to print in the allocated space. */
		va_start(ap, fmt);
		n = vsnprintf (buff, size, fmt, ap);
		va_end(ap);
		if (n > -1 && n < size) {
			break;
		}
		else {
			/* Else try again with more space. */
			if (n > -1)    /* glibc 2.1 */
				size = n+1; /* precisely what is needed */
			else           /* glibc 2.0 */
				size *= 2;  /* twice the old size */
			if ((buff = realloc (buff, size)) == NULL)
				return l;
		}
	}
	if (buff != NULL) {
		PVERB(PINFO,"# Send: %s\n", buff);
#ifdef DEMO_MODE
		l = write(dev, buff, n);
		write(dev, "\n", 1);

#else
		l = write_func(dev, buff, n);
#endif

#ifdef VERBOSE_MODE
		if (l & ERR /*&& !(l & TIMO)*/) {
			SYSLOG(LOG_ERR,"Error sending: %s [ibstat: 0x%x]\n", 
			      buff, l);
		}
#endif
		free(buff);
	}

	return l;
}


/* Wait for service request */
static int cc_wsq(int dev)
{
	u_char spr;

	while (!(ibrsp(dev, &spr) & ERR)) {
#ifdef DEBUG_MODE
		PVERB(PINFO," Service request ret: 0x%x\n", spr);
#endif
		if (spr == CMD_OK) {
			break;
		}else if (spr == CMD_ERROR) {
			return -1;
		}
	}
	
	return 0;
}


static int __get_int_from_cc(int dev, char *cmd, int *val)
{
	char ctmp[RW_BUF_SIZE];

	if ( my_ibwrite_generic(ibwrt, dev, "@%c", *cmd) & ERR) 
		return -1;
	if (cc_wsq(dev) != 0) 
		return -1;
	memset(ctmp, 0, RW_BUF_SIZE);
	if (ibrd(dev, ctmp, RW_BUF_SIZE) & ERR) {
		return -1;
	}else{
		*val = atoi(ctmp);

	}		

	return 0;
}

int get_int_from_cc(int dev, char *cmd, int *val)
{
	int v = -1;
	int ret; 
	int retry = 0;

	do {
		*val = v;
		ret = __get_int_from_cc(dev, cmd, &v);
	}while( (v != *val) && (ret != 0) && retry++ < CC_RETRY_GET);
	
	return ret;
}

int get_str_from_cc(int dev, char *cmd, char *str, int len)
{
	if ( my_ibwrite_generic(ibwrt, dev, "@%s", cmd) & ERR) 
		return -1;
	if (cc_wsq(dev) != 0) 
		return -1;
	memset(str, 0, len);
	ibtmo(dev, T1s);
	if (ibrd(dev, str, len) & ERR) {
		str = NULL;
		ibtmo(dev, CC_TIME_OUT);
		return -1;
	}
	PVERB(PINFO,"get_str_from_cc: done.\n");
	ibtmo(dev, CC_TIME_OUT);
	
	return 0;
}


int put_int_to_cc(int dev, char *cmd, int val)
{
	if ( my_ibwrite_generic(ibwrt, dev, "!%c%d", *cmd, val) & ERR) 
		return -1;
	if (cc_wsq(dev) != 0) 
		return -1;

	return 0;
}


int put_str_to_cc(int dev, char *cmd, char *str)
{
	if ( my_ibwrite_generic(ibwrt, dev, "!%c%s", *cmd, str) & ERR)
		return -1;
	if (cc_wsq(dev) != 0) 
		return -1;

	return 0;
}


int action_to_cc(int dev, char *cmd)
{
	if ( my_ibwrite_generic(ibwrt, dev, ":%s", cmd) & ERR) 
		return -1;
	if (cc_wsq(dev) != 0) 
		return -1;

	return 0;
}


int send_data_to_cc(int dev, char *data, int len)
{
	ibtmo(dev, T1s);
	ibwrt(dev, data, len);
	ibtmo(dev, CC_TIME_OUT);

	return ibcnt >> 1;
}


int rcv_data_from_cc(int dev, char *data, int *len)
{
	int rcv;
#if 0
	ibtmo(dev, T10s);
	ibrd(dev, data, *len);
	rcv = ibcnt;
	ibtmo(dev, CC_TIME_OUT);
	*len = rcv;
#endif
	int i;

	for(i=0; i < *len;i++)
		data[i] = 1;// & 0x0fff;

	rcv = *len;


	return rcv;
}
